using System;
using System.Collections;
using Server.Items;
using Server.Targeting;
using Server.Mobiles;
using Server.Commands;



namespace Server.Spells.FSB
{
	public class FSBCommands
	{
		public static void Initialize()
		{
			CommandSystem.Register( "FillSchools", AccessLevel.GameMaster, new CommandEventHandler( FillSchools_OnCommand ) );
		}

		[Usage( "FillSchools" )]
		[Description( "Fills the current Schools of the targeted Full Spellbook." )]
		public static void FillSchools_OnCommand( CommandEventArgs e )
		{
			e.Mobile.BeginTarget( -1, false, TargetFlags.None, new TargetCallback( FillSchools_OnTarget ) );
			e.Mobile.SendMessage( "Which Full Spellbook would you like to fill?" );
		}

		private static void FillSchools_OnTarget( Mobile from, object obj )
		{
			if( obj is TomeOfKnowledge )
			{
				TomeOfKnowledge book = (TomeOfKnowledge)obj;

				for( int i = 2; i < FSBSchoolList.SchoolList.Count; i++ )
				{
					FSBSchoolInfo si = (FSBSchoolInfo)FSBSchoolList.SchoolList[i];
					if( si == null )
						continue;

					if( book.GetFlag( si.School ) )
					{
						string[] Ranges = si.Range.Split();

						for( int x = 0; x < Ranges.Length; x++ )
						{
							int spellID = -1;

							try
							{
								spellID = Int32.Parse( Ranges[x] );
								if( book.ContainsSpell( spellID ) )
									continue;
								book.SpellContents.Add( spellID );
							}
							catch
							{
								Console.WriteLine( string.Format( "Bad SpellID in {0} Ranges : {1}", book.CurrentSchool.Name, Ranges[x] ) );
							}
						}
					}
				}

				from.SendMessage( "The Schools have been filled." );
				CommandLogging.WriteLine( from, "{0} {1} filling Full Spellbook Schools {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( book ) );
			}
			else
			{
				from.BeginTarget( -1, false, TargetFlags.None, new TargetCallback( FillSchools_OnTarget ) );
				from.SendMessage( "That is not a Full Spellbook. Try again." );
			}
		}
	}
}
