using System;
using Server.Network;
using Server.Items;
using Server.Gumps;
using Server.Misc;
using System.Collections;
using Server.Spells;
using Server.Engines.Craft;
using Server.ACC.CM;
using Server.LucidNagual;
using Server.Enums;
using Server.Mobiles;


namespace Server.Spells.FSB
{
	#region Flags
	[Flags]
	public enum BookFlag
	{
		None        =   0x00000000,
		IsFull      =   0x00000001,
		Closable    =   0x00000002,
		ShowRegs    =   0x00000004,
		KeepRegs	=	0x00000008,
	}
	#endregion      //Flags
	
	public class TomeOfKnowledge : Item, ICraftable
	{
		#region Variables
		internal Mobile m_Owner;
		
		public FSBSchoolInfo CurrentSchool;
		public ArrayList DragableIcons;
		public ArrayList SpellContents;
		public ArrayList Keywords;
		public Hashtable Regs;
		public RegRefTimer Refresher;
		private FSBSchoolFlags m_Schools;
		private SchoolFlag m_SchoolFlags;
		private BookFlag m_BookFlags;
		private SlayerName m_Slayer;
		private AosAttributes m_AosAttributes;
		private AosSkillBonuses m_AosSkillBonuses;
		
		public int StartX;
		public int StartY;
		public int Skin;
		public int RegX;
		public int RegY;
		public int SpellsOffset;
		public int OptionsOffset;
		public int CurrentSpell;
		
		public SchoolFlag   SchoolFlags{    get{ return m_SchoolFlags; }    set{ m_SchoolFlags = value; } }
		public BookFlag     BookFlags{      get{ return m_BookFlags; }      set{ m_BookFlags = value; } }
		
		public Type[] RegTypes{ get{ return FSBSchoolFlags.m_RegTypes; } }
		public Type[] NomadTypes{ get{ return FSBSchoolFlags.m_AllScrollTypes; } }
		public Type[] MageryTypes{ get{ return Loot.RegularScrollTypes; } }
		public Type[] NecroTypes{ get{ return Loot.NecromancyScrollTypes; } }
		public Type[] AvatarTypes{ get{ return Loot.AvatarScrollTypes; } }
		public Type[] BardTypes{ get{ return Loot.SongScrollTypes; } }
		public Type[] ClericTypes{ get{ return Loot.ClericScrollTypes; } }
		public Type[] DruidTypes{ get{ return Loot.DruidScrollTypes; } }
		//public Type[] FarmTypes{ get{ return Loot.FarmerScrollTypes; } }
		public Type[] RangerTypes{ get{ return Loot.RangerScrollTypes; } }
		public Type[] RogueTypes{ get{ return Loot.RogueScrollTypes; } }
		public Type[] StaffTypes{ get{ return FSBSchoolFlags.m_AncientScrollTypes; } }
		public Type[] UndeadTypes{ get{ return Loot.UndeadScrollTypes; } }
		#endregion      //Variables
		
		#region Properties
		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Owner{ get{ return m_Owner; } set{ m_Owner = value; } }
		
		[CommandProperty( AccessLevel.GameMaster )]
		public FSBSchoolFlags Schools{ get{ return m_Schools; } set{} }
		
		[CommandProperty( AccessLevel.GameMaster )]
		public bool IsFull{ get{ return GetFlag( BookFlag.IsFull ); } set{ SetFlag( BookFlag.IsFull, value ); } }
		
		[CommandProperty( AccessLevel.GameMaster )]
		public AosAttributes Attributes{ get{ return m_AosAttributes; } set{} }
		
		[CommandProperty( AccessLevel.GameMaster )]
		public AosSkillBonuses SkillBonuses{ get{ return m_AosSkillBonuses; } set{} }
		
		[CommandProperty( AccessLevel.GameMaster )]
		public SlayerName Slayer{ get{ return m_Slayer; } set{ m_Slayer = value; InvalidateProperties(); } }
		#endregion      //Properties
		
		[Constructable]
		public TomeOfKnowledge() : base( 3834 )
		{
			Movable = true;
			Weight = 1;
			LootType = LootType.Newbied;
			Layer = Layer.OneHanded;
			Hue = 1367;
			
			m_Owner = null;
			m_Schools = new FSBSchoolFlags( this );
			
			m_AosAttributes = new AosAttributes( this );
			m_AosSkillBonuses = new AosSkillBonuses( this );
			
			Name = "Ancient Tome of Knowledge";
			
			StartX = /*StartY =*/ Skin = RegX = RegY = 0;
			StartY = 100;
			
			DragableIcons = new ArrayList();
			SpellContents = new ArrayList();
			Keywords = new ArrayList();
			Regs = new Hashtable();
			
			Refresher = new RegRefTimer( m_Owner, this );
			
			CurrentSchool = (FSBSchoolInfo)FSBSchoolList.SchoolList[0];
			CurrentSpell = -1;
			
			FSBSettings.InitSchools( this );
		}
		
		public TomeOfKnowledge( Serial serial ) : base( serial )
		{
		}
		
		#region Overrides
		public override void OnDoubleClick( Mobile from )
		{
			if ( ASSettings.EnableAllSystems || ASSettings.EnableClassSystem )
			{
				if ( from.AccessLevel > AccessLevel.Counselor )
					OwnerCheck ( from );
				
				else
				{
					ModuleCheck ( from );
					ClassCheck ( from );
					OwnerCheck ( from );
				}
			}
			else
				OwnerCheck ( from );
		}
		
		public override bool HandlesOnSpeech{ get{ return true; } }
		
		public override void OnSpeech( SpeechEventArgs e )
		{
			base.OnSpeech( e );
			
			if ( !e.Handled && e.Mobile == m_Owner && Keywords != null )
			{
				for( int i = 0; i < Keywords.Count; i++ )
				{
					FSBKeyword key = Keywords[i] as FSBKeyword;
					if( key == null )
						continue;
					
					FSBSpellInfo si = FSBSpellList.FindSpellInfo( key.Spell );
					if( si == null )
						continue;
					
					if( GetFlag( si.School ) || GetFlag( BookFlag.IsFull ) )
					{
						string keyword = key.Keyword;
						
						if( keyword == null || (keyword = keyword.Trim()).Length == 0 )
							continue;
						
						if( Insensitive.Equals( keyword, e.Speech ) )
						{
							int spell = key.Spell;
							
							if( spell == -1 )
								continue;
							
							Spell spellToCast = SpellRegistry.NewSpell( spell, m_Owner, null );
							if( spellToCast != null )
							{
								spellToCast.Cast();
							}
							else
							{
								m_Owner.SendMessage( "Somehow that keyword has a bad Spell Registry.  Removing Keyword" );
								Keywords.RemoveAt(i);
							}
						}
					}
				}
			}
		}
		
		public override void GetProperties( ObjectPropertyList list )
		{
			base.GetProperties( list );
			
			if( m_Slayer != SlayerName.None )
				list.Add( SlayerGroup.GetEntryByName( m_Slayer ).Title );
			
			m_AosSkillBonuses.GetProperties( list );
			int prop;
			if( (prop = m_AosAttributes.LowerManaCost) != 0 )
				list.Add( 1060433, prop.ToString() ); // lower mana cost ~1_val~%
			if ( (prop = m_AosAttributes.RegenMana) != 0 )
				list.Add( 1060440, prop.ToString() ); // mana regeneration ~1_val~
			if ( (prop = m_AosAttributes.LowerRegCost) != 0 )
				list.Add( 1060434, prop.ToString() ); // lower reagent cost ~1_val~%
			if ( (prop = m_AosAttributes.CastRecovery) != 0 )
				list.Add( 1060412, prop.ToString() ); // faster cast recovery ~1_val~
			if ( (prop = m_AosAttributes.CastSpeed) != 0 )
				list.Add( 1060413, prop.ToString() ); // faster casting ~1_val~
			if ( (prop = m_AosAttributes.BonusInt) != 0 )
				list.Add( 1060432, prop.ToString() ); // intelligence bonus ~1_val~
			if ( (prop = m_AosAttributes.BonusMana) != 0 )
				list.Add( 1060439, prop.ToString() ); // mana increase ~1_val~
			if ( (prop = m_AosAttributes.SpellDamage) != 0 )
				list.Add( 1060483, prop.ToString() ); // spell damage increase ~1_val~%
			
			if( Regs != null && Regs.Count > 0 )
			{
				string entry = "";
				
				for( int i = 0; i < RegTypes.Length; i++ )
				{
					if( Regs.ContainsKey( RegTypes[i] ) && (int)Regs[RegTypes[i]] > 0 )
						entry += String.Format( "\n{0} : {1}", RegTypes[i].Name, (int)Regs[RegTypes[i]] );
				}
				
				list.Add( 1060658, "--------------------------------------\nReagents \t{0}", entry );
			}
		}
		
		public override bool OnDragDrop( Mobile from, Item dropped )
		{
			if( m_Owner != null )
			{
				if( dropped is Gold )
				{
					from.TithingPoints += dropped.Amount;
					dropped.Delete();
					PrivateOverheadMessage( MessageType.Whisper, 0, false, "Converted the gold into Tithe.", from.NetState );
					return true;
				}
				else if( dropped is Spellbook )
				{
					if ( AbsorbBook( from, dropped ) )
						return true;
					
					return false;
				}
				else if( dropped is SpellScroll && dropped.Amount == 1 )
				{
					if ( AddScroll( from, dropped ) )
						return true;
					
					return false;
				}
				/*else
				{
					PrivateOverheadMessage( MessageType.Whisper, 0, false, "That item cannot be placed in this book.", from.NetState );
					return false;
				}*/
				
				foreach ( Type type in RegTypes )
				{
					if ( dropped.GetType() == type )
					{
						AddReg( type, dropped.Amount );
						dropped.Delete();
						PrivateOverheadMessage( MessageType.Whisper, 0, false, "Added the reagent", from.NetState );
						InvalidateProperties();
						return true;
					}
				}
				PrivateOverheadMessage( MessageType.Whisper, 0, false, "That item cannot be placed in this book.", from.NetState );
				//PrivateOverheadMessage( MessageType.Whisper, 0, false, "That's not a reagent!", from.NetState );
				return false;
			}
			else
			{
				PrivateOverheadMessage( MessageType.Whisper, 0, false, "I can't accept anything til someone owns me!", from.NetState );
				return false;
			}
		}
		
		public override bool OnEquip( Mobile from )
		{
			if( m_Owner == null || m_Owner != from )
			{
				from.SendMessage( "This Tome does not belong to you!" );
				return false;
			}
			
			int intBonus = m_AosAttributes.BonusInt;
			m_AosSkillBonuses.AddTo( from );
			
			if ( intBonus != 0 )
			{
				Mobile m = from;
				
				string modName = this.Serial.ToString();
				
				if ( intBonus != 0 )
					m.AddStatMod( new StatMod( StatType.Int, modName + "Int", intBonus, TimeSpan.Zero ) );
			}
			
			return true;
		}
		
		public override void OnRemoved( object parent )
		{
			if( Core.AOS && parent is Mobile )
			{
				Mobile from = (Mobile)parent;
				
				m_AosSkillBonuses.Remove();
				
				string modName = this.Serial.ToString();
				
				from.RemoveStatMod( modName + "Int" );
				
				from.CheckStatTimers();
			}
		}
		
		public override bool AllowEquipedCast( Mobile from )
		{
			if( m_Owner != null && m_Owner == from )
				return true;
			return false;
		}
		#endregion      //Overrides
		
		#region ClassMethods
		public bool GetFlag( SchoolFlag flag ) { return( (m_SchoolFlags & flag) != 0 ); }
		public void SetFlag( SchoolFlag flag, bool value )
		{
			if ( value ) m_SchoolFlags |= flag;
			else m_SchoolFlags &= ~flag;
		}
		
		public bool GetFlag( BookFlag flag ) { return( (m_BookFlags & flag) != 0 ); }
		public void SetFlag( BookFlag flag, bool value )
		{
			if( value ) m_BookFlags |= flag;
			else m_BookFlags &= ~flag;
		}
		
		#region CheckMethods
		public bool ModuleCheck( Mobile from ) //Purpose is to check for and create a module if non-existant.
		{
			PlayerMobile pm = from as PlayerMobile;
			PlayerModule mod = pm.PlayerModule; //Has a check to create non-existing module.
			
			return true;
		}
		
		public bool ClassCheck( Mobile from )
		{
			PlayerMobile pm = from as PlayerMobile;
			PlayerModule mod = pm.PlayerModule; 
			
			if ( from.AccessLevel < AccessLevel.Counselor && mod.PlayerClasses == PlayerClasses.None )
			{
				from.SendGump( new ClassGump( from ) );
				from.SendMessage( "You need to select a class." );
				return false;
			}
			
			return true;
		}
		
		public bool OwnerCheck( Mobile from )
		{
			if( m_Owner == null )
			{
				m_Owner = from;
				from.PlaySound( 0x1F7 );
				from.AddToBackpack( this );
				Name = m_Owner.Name + "'s Ancient Tome of Knowledge";
				from.SendMessage( "The markings on the tome change..." );
				
				if( FSBSettings.ClassEnabled )
					FSBSettings.InitRestrictions( this, from );
				
				return true;
			}
			
			else if( m_Owner == from )
			{
				//if( FSBSettings.ClassEnabled )
				//	FSBSettings.InitRestrictions( this, from );
				
				from.CloseGump( typeof( SpellIconGump ) );
				from.CloseGump( typeof( TomeOfKnowledgeGump ) );
				
				for( int i = 0; i < DragableIcons.Count; i++ )
				{
					IconInfo ii = (IconInfo)DragableIcons[i];
					if( ii != null )
					{
						from.SendGump( new SpellIconGump( this, from, ii.X, ii.Y, ii.Icon, ii.Spell, ii.Background ) );
					}
				}
				from.SendGump( new TomeOfKnowledgeGump( from, this ) );
				
				return true;
			}
			
			else
			{
				from.SendMessage( "The tome does not allow you to view its contents" );
				return false;
			}
		}
		
		public bool BookCheck( Mobile from, Item dropped )
		{
			PlayerMobile pm = from as PlayerMobile;
			PlayerModule mod = pm.PlayerModule; 			
			
			string msg = "The book will not drop. You do not posses the training to pass this book through.";
			string allowed = "You successfully added the book.";
			
			if ( from.AccessLevel > AccessLevel.Counselor )
			{
				if ( dropped is SpellweavingBook )
				{
					from.SendMessage( "That book is not allowed til this is updated for ML!" );
					return false;
				}
				else if ( dropped is BookOfFarming )
				{
					from.SendMessage( "That book is not allowed til this is updated for Farming!" );
					return false;
				}
				else
				{
					from.SendMessage(32, "Due to your godly powers, your are able to add the book!");
					return true;
				}
			}
			
			if ( ASSettings.EnableAllSystems || ASSettings.EnableClassSystem )
			{
				ClassCheck ( from );
				
				if ( mod.PlayerClasses == PlayerClasses.Nomad )
					return true;
				
				else if ( dropped is SpellweavingBook )
				{
					from.SendMessage( "That book is not allowed til this is updated for ML!" );
					return false;
				}
				
				else if ( dropped is BookOfFarming )
				{
					from.SendMessage( "That book is not allowed til this is updated for Farming!" );
					return false;
				}
				
				else if ( dropped is AncientSpellbook )
				{
					from.SendMessage( "Players are not allowed to have that book!" );
					dropped.Delete();
					return false;
				}
				
				else if ( mod.PlayerClasses == PlayerClasses.Paladin && dropped is BookOfChivalry || dropped is AvatarSpellbook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Necromancer && dropped is NecromancerSpellbook || dropped is UndeadSpellbook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Tamer && dropped is DruidSpellbook || dropped is SongBook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Mage && dropped is Spellbook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Druid && dropped is DruidSpellbook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Cleric && dropped is ClericSpellbook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Ranger && dropped is RangerSpellbook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Rogue && dropped is RogueSpellbook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Bard && dropped is SongBook )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Ninja && dropped is BookOfNinjitsu )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Samurai && dropped is BookOfBushido )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Arcanist && dropped is SpellweavingBook )
					return true;
				
				else
				{
					from.SendMessage( "{0}", msg );
					return false;
				}
			}
			else
			{
				from.SendMessage( "{0}", allowed );
				return true;
			}
		}
		#endregion CheckMethods
		
		public bool AbsorbBook( Mobile from, Item dropped )
		{
			Spellbook book = (Spellbook)dropped;
			FSBSpellInfo si = FSBSpellList.FindSpellInfo( book.BookOffset );
			
			if( book == null || si == null )
				return false;
			
			if ( BookCheck( from, dropped ) == true )
			{
				
				if( !FSBSettings.EnableOnDrop && !GetFlag( si.School ) && !GetFlag( BookFlag.IsFull ) )
				{
					PrivateOverheadMessage( MessageType.Whisper, 0, false, "I do not know that form of magic!", from.NetState );
					return false;
				}
				
				if( FSBSettings.EnableOnDrop )
				{
					if( !FSBSettings.ReDrop && GetFlag( si.School ) )
					{
						PrivateOverheadMessage( MessageType.Whisper, 0, false, "I already know that form of magic!", from.NetState );
						return false;
					}
					
					SetFlag( si.School, true );
				}
				
				for( int i = book.BookOffset; i < book.BookOffset + book.BookCount; i++ )
				{
					if( !book.HasSpell( i ) )
						continue;
					
					if( ContainsSpell( i ) )
						continue;
					
					FSBSpellInfo addSI = FSBSpellList.FindSpellInfo( i );
					if( addSI == null || (!GetFlag( si.School ) && !GetFlag( BookFlag.IsFull )) )
						continue;
					
					SpellContents.Add( i );
				}
				
				PrivateOverheadMessage( MessageType.Whisper, 0, false, "I have absorbed the essence of that book!", from.NetState );
				return true;
			}
			else
				return false;
		}
		
		public bool ScrollCheck( Mobile from, Item dropped )
		{
			PlayerMobile pm = from as PlayerMobile;
			PlayerModule mod = pm.PlayerModule; 			
			
			//string msg = "The scroll will not drop. You do not posses the training to pass this scroll through.";
			//string allowed = "You successfully added the scroll.";
			SpellScroll scroll = ( SpellScroll )dropped;
			
			if ( from.AccessLevel > AccessLevel.Counselor )
			{
				if ( from.AccessLevel > AccessLevel.Seer )
				{
					foreach ( Type type in StaffTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else
				{
					from.SendMessage(32, "Due to your godly powers, your are able to add the scroll!");
					return true;
				}
			}
			
			if ( DataCenter.EnableAllSystems || DataCenter.EnableClassSystem )
			{
				ClassCheck ( from );
				
				if ( mod.PlayerClasses == PlayerClasses.Nomad )
					return true;
				
				else if ( mod.PlayerClasses == PlayerClasses.Mage )
				{
					foreach ( Type type in MageryTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Paladin )
				{
					foreach ( Type type in AvatarTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Necromancer )
				{
					foreach ( Type type in NecroTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Necromancer )
				{
					foreach ( Type type in UndeadTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Bard )
				{
					foreach ( Type type in BardTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Cleric )
				{
					foreach ( Type type in ClericTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Druid )
				{
					foreach ( Type type in DruidTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Ranger )
				{
					foreach ( Type type in RangerTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Rogue )
				{
					foreach ( Type type in RogueTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Tamer )
				{
					foreach ( Type type in BardTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Tamer )
				{
					foreach ( Type type in DruidTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else if ( mod.PlayerClasses == PlayerClasses.Nomad )
				{
					foreach ( Type type in NomadTypes )
					{
						if ( dropped.GetType() == type )
							return true;
						
						else
							return false;
					}
				}
				else
					return false;
			}
			return false;
		}
		
		
		public bool AddScroll( Mobile from, Item dropped )
		{
			if( SpellContents == null )
				SpellContents = new ArrayList();
			
			SpellScroll scroll = (SpellScroll)dropped;
			FSBSpellInfo si = FSBSpellList.FindSpellInfo( scroll.SpellID );
			
			if( si == null )
			{
				PrivateOverheadMessage( MessageType.Whisper, 0, false, "Unkown Spell.", from.NetState );
				return false;
			}
			else if( !GetFlag( si.School ) && !GetFlag( BookFlag.IsFull ) )
			{
				PrivateOverheadMessage( MessageType.Whisper, 0, false, "Cannot except that scroll if the tome doesn't contain the proper spellbook!", from.NetState );
				return false;
			}
			
			if( ContainsSpell( scroll.SpellID ) )
			{
				from.SendLocalizedMessage( 500179 ); // That spell is already present in that spellbook.
				return false;
			}
			
			ScrollCheck ( from, scroll );
			
			SpellContents.Add( scroll.SpellID );
			PrivateOverheadMessage( MessageType.Whisper, 0, false, "Added that spell to my memory.", from.NetState );
			
			scroll.Delete();
			
			return true;
		}
		
		public bool ContainsSpell( int spellID )
		{
			bool ret = false;
			
			if( SpellContents == null )
			{
				SpellContents = new ArrayList();
				return false;
			}
			
			FSBSpellInfo si = FSBSpellList.FindSpellInfo( spellID );
			
			if( si == null )
				return false;
			
			for( int i = 0; i < SpellContents.Count; i++ )
			{
				if( (int)SpellContents[i] == spellID && GetFlag( si.School ) )
					ret = true;
			}
			
			return ret;
		}
		
		public void AddKeyword( string keyword, int spellID )
		{
			if( Keywords == null || Keywords.Count <= 0 )
			{
				Keywords = new ArrayList();
				Keywords.Add( new FSBKeyword( spellID, keyword ) );
			}
			
			else
			{
				if( !CheckKeyword( keyword, spellID ) )
				{
					m_Owner.SendMessage( "That Keyword is already assigned to another spell." );
					return;
				}
				else
				{
					if( keyword == null || keyword.Length == 0 )
					{
						m_Owner.SendMessage( "Removed that keyword" );
						return;
					}
					Keywords.Add( new FSBKeyword( spellID, keyword ) );
					m_Owner.SendMessage( "Keyword has been set : " + keyword );
				}
			}
		}
		
		public bool CheckKeyword( string keyword, int spellID )
		{
			if( Keywords == null )
				return true;
			
			for( int i = 0; i < Keywords.Count; i++ )
			{
				FSBKeyword key = (FSBKeyword)Keywords[i];
				if( key == null )
					continue;
				
				if( key.Spell == spellID )
				{
					Keywords.RemoveAt(i);
					continue;
				}
				
				if( Insensitive.Equals( keyword, key.Keyword ) )
					return false;
			}
			
			return true;
		}
		
		public void AddReg( Type type, int amount )
		{
			if ( Regs == null )
			{
				Regs = new Hashtable();
				Regs.Add (type,amount);
				return;
			}
			
			if ( Regs.ContainsKey(type) )
			{
				Regs[type] = (int)Regs[type] + amount;
				return;
			}
			else
				Regs.Add(type,amount);
		}
		
		public bool ConsumeReg( Type[] type, int[] amount )
		{
			try
			{
				for ( int i=0; i < type.Length; i++ )
				{
					if ( !ConsumeReg( type[i], amount[i] ) )
						return false;
				}
			}
			catch
			{
				Console.WriteLine("Error occured in consuming reagent from Full Spellbook.");
				return false;
			}
			ConsumeReagents( type, amount );
			return true;
		}
		
		private bool ConsumeReg( Type type, int amount )
		{
			if ( !Regs.ContainsKey(type) )
				return false;
			
			if ( (int)Regs[type] < amount )
				return false;
			return true;
		}
		
		private void ConsumeReagents( Type[] type, int[] amount )
		{
			for (int i=0; i < type.Length; i++)
				Regs[type[i]] = (int)Regs[type[i]] - amount[i];
		}
		
		private void PrivateOverheadMessage( MessageType type, int hue, bool ascii, string text, NetState state )
		{
			if( state == null ) return;
			
			if( ascii )
				state.Send( new AsciiMessage( Serial, ItemID, type, hue, 3, Name, text ) );
			else
				state.Send( new UnicodeMessage( Serial, ItemID, type, hue, 3, "ENU", Name, text ) );
		}
		
		#region ICraftable Members
		private const int MaxProperties = 32;
		private static BitArray m_Props = new BitArray( MaxProperties );
		private static int[] m_Possible = new int[MaxProperties];
		
		public static int GetUniqueRandom( int count )
		{
			int avail = 0;
			
			for ( int i = 0; i < count; ++i )
			{
				if ( !m_Props[i] )
					m_Possible[avail++] = i;
			}
			
			if ( avail == 0 )
				return -1;
			
			int v = m_Possible[Utility.Random( avail )];
			
			m_Props.Set( v, true );
			
			return v;
		}
		
		public int OnCraft( int quality, bool makersMark, Mobile from, CraftSystem craftSystem, Type typeRes, BaseTool tool, CraftItem craftItem, int resHue )
		{
			m_Props.SetAll( false );
			
			double theirSkill = from.Skills[SkillName.Magery].Base;
			
			if ( quality > 1 )
			{
				int count = 0;
				
				if ( theirSkill >= 120.0 )
					count = Utility.RandomMinMax( 2, 3 );
				
				else if ( theirSkill >= 110.0 )
					count = Utility.RandomMinMax( 2, 2 );
				
				else if ( theirSkill >= 100.0 )
					count = Utility.RandomMinMax( 1, 3 );
				
				else if ( theirSkill >= 90.0 )
					count = Utility.RandomMinMax( 1, 2 );
				
				else if ( theirSkill >= 80.0 )
					count = 1;
				
				int attributeCount = count;
				
				for ( int i = 0; i < attributeCount; ++i )
				{
					int random = GetUniqueRandom( 13 );
					int bon = Utility.RandomMinMax( 1, 8 );
					int fcr = Utility.RandomMinMax( 1, 3 );
					int spdm = Utility.RandomMinMax( 1, 12 );
					int skb = Utility.RandomMinMax( 1, 15 );
					
					if ( random == -1 )
						break;
					
					switch ( random )
					{
							case 0: Attributes.BonusMana = bon; break;
							case 1: Attributes.BonusInt = bon; break;
							case 2: Attributes.CastSpeed = 1; break;
							case 3: Attributes.CastRecovery = fcr; break;
							case 4: Attributes.SpellDamage = spdm; break;
							case 5: SkillBonuses.SetValues(0, SkillName.Magery, skb ); break;
							case 6: SkillBonuses.SetValues(1, SkillName.Meditation, skb ); break;
							case 7: SkillBonuses.SetValues(2, SkillName.EvalInt, skb ); break;
							case 8: SkillBonuses.SetValues(3, SkillName.MagicResist, skb ); break;
							case 9: Attributes.LowerRegCost = skb; break;
							case 10: Attributes.LowerManaCost = skb; break;
							case 11: Attributes.RegenMana = skb; break;
						case 12:
							{
								SlayerName slayer = SlayerName.None;
								if ( Core.AOS )
									slayer = BaseRunicTool.GetRandomSlayer();
								else
									slayer = SlayerGroup.GetLootSlayerType( from.GetType() );
								Slayer = slayer;
								break;
							}
					}
				}
			}
			return quality;
		}
		#endregion		//ICraftable Members
		#endregion      //ClassMethods
		
		#region DeSerialization
		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 ); //version
			
			writer.Write( (Mobile)m_Owner );
			
			writer.WriteEncodedInt( (int)m_Slayer );
			m_AosAttributes.Serialize( writer );
			m_AosSkillBonuses.Serialize( writer );
			
			writer.Write( (int)StartX );
			writer.Write( (int)StartY );
			writer.Write( (int)Skin );
			writer.Write( (int)RegX );
			writer.Write( (int)RegY );
			writer.Write( (int)SpellsOffset );
			
			writer.Write( (int)DragableIcons.Count );
			for( int i = 0; i < DragableIcons.Count; i++ )
				((IconInfo)DragableIcons[i]).Serialize( writer );
			
			writer.Write( (int)SpellContents.Count );
			for( int i = 0; i < SpellContents.Count; i++ )
				writer.Write( (int)SpellContents[i] );
			
			writer.Write( (int)Keywords.Count );
			for( int i = 0; i < Keywords.Count; i++ )
				((FSBKeyword)Keywords[i]).Serialize( writer );
			
			writer.Write( (int)Regs.Count );
			foreach( DictionaryEntry de in Regs )
			{
				writer.Write( ((Type)de.Key).Name );
				writer.Write( (int)de.Value );
			}
			
			writer.Write( (int)m_SchoolFlags );
			writer.Write( (int)m_BookFlags );
			
			CurrentSchool.Serialize( writer );
			writer.Write( (int)CurrentSpell );
		}
		
		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
			
			DragableIcons = new ArrayList();
			SpellContents = new ArrayList();
			Keywords = new ArrayList();
			Regs = new Hashtable();
			OptionsOffset = 0;
			
			m_Owner = reader.ReadMobile();
			
			m_Slayer = (SlayerName)reader.ReadEncodedInt();
			m_AosAttributes = new AosAttributes( this, reader );
			m_AosSkillBonuses = new AosSkillBonuses( this, reader );
			
			if( Core.AOS && Parent is Mobile )
				m_AosSkillBonuses.AddTo( (Mobile)Parent );
			int intBonus = m_AosAttributes.BonusInt;
			if( Parent is Mobile && (intBonus != 0) )
			{
				Mobile m = (Mobile)Parent;
				string modName = Serial.ToString();
				
				m.AddStatMod( new StatMod( StatType.Int, modName + "Int", intBonus, TimeSpan.Zero ) );
			}
			if( Parent is Mobile )
				((Mobile)Parent).CheckStatTimers();
			
			StartX = reader.ReadInt();
			StartY = reader.ReadInt();
			Skin = reader.ReadInt();
			RegX = reader.ReadInt();
			RegY = reader.ReadInt();
			SpellsOffset = reader.ReadInt();
			
			int count = reader.ReadInt();
			for( int i = 0; i < count; i++ )
				DragableIcons.Add( new IconInfo( reader ) );
			
			count = reader.ReadInt();
			for( int i = 0; i < count; i++ )
				SpellContents.Add( reader.ReadInt() );
			
			count = reader.ReadInt();
			for( int i = 0; i < count; i++ )
				Keywords.Add( new FSBKeyword( reader ) );
			
			count = reader.ReadInt();
			for( int i = 0; i < count; i++ )
				Regs.Add( ScriptCompiler.FindTypeByName(reader.ReadString()), reader.ReadInt() );
			
			m_SchoolFlags = (SchoolFlag)reader.ReadInt();
			m_BookFlags = (BookFlag)reader.ReadInt();
			
			CurrentSchool = new FSBSchoolInfo( reader );
			CurrentSpell = reader.ReadInt();
			
			Refresher = new RegRefTimer( m_Owner, this );
			m_Schools = new FSBSchoolFlags( this );
		}
		#endregion      //DeSerialization
	}
}
