using System;
using Server.Misc;

namespace Server.Items
{ 
	public class ChestOfTheMage : LeatherChest 
	{ 
		private SkillMod m_SkillMod0; 
		private SkillMod m_SkillMod1; 
		private SkillMod m_SkillMod2; 
		private SkillMod m_SkillMod3; 
		private StatMod m_StatMod0; 
		
		[Constructable] 
		public ChestOfTheMage()
		{ 
			Name = "Chest Of The Mage"; 
			Hue = 0x484;
			DefineMods();
		} 

		private void DefineMods()
		{
			m_SkillMod0 = new DefaultSkillMod( SkillName.Magery, true, 2 ); 
			m_SkillMod1 = new DefaultSkillMod( SkillName.EvalInt, true, 2 ); 
			m_SkillMod2 = new DefaultSkillMod( SkillName.MagicResist, true, 2 ); 
			m_SkillMod3 = new DefaultSkillMod( SkillName.Meditation, true, 2 ); 
			m_StatMod0 = new StatMod( StatType.Int, "MageChest", 2, TimeSpan.Zero ); 
		}

		private void SetMods( Mobile wearer )
		{			
			wearer.AddSkillMod( m_SkillMod0 ); 
			wearer.AddSkillMod( m_SkillMod1 ); 
			wearer.AddSkillMod( m_SkillMod2 ); 
			wearer.AddSkillMod( m_SkillMod3 ); 
			wearer.AddStatMod( m_StatMod0 ); 
		}

		public override bool OnEquip( Mobile from ) 
		{ 
			SetMods( from );
			return true;  
		} 

		public override void OnRemoved( object parent ) 
		{ 
			if ( parent is Mobile ) 
			{ 
				Mobile m = (Mobile)parent;
				m.RemoveStatMod( "MageChest" ); 

				if ( m.Hits > m.HitsMax )
					m.Hits = m.HitsMax; 

				if ( m_SkillMod0 != null ) 
					m_SkillMod0.Remove(); 

				if ( m_SkillMod1 != null ) 
					m_SkillMod1.Remove(); 

				if ( m_SkillMod2 != null ) 
					m_SkillMod2.Remove(); 

				if ( m_SkillMod3 != null ) 
					m_SkillMod3.Remove(); 
			} 
		} 

		public override void OnSingleClick( Mobile from ) 
		{ 
			this.LabelTo( from, Name ); 
		} 

		public ChestOfTheMage( Serial serial ) : base( serial ) 
		{ 
			DefineMods();
			
			if ( Parent != null && this.Parent is Mobile ) 
				SetMods( (Mobile)Parent );
		} 

		public override void Serialize( GenericWriter writer ) 
		{ 
			base.Serialize( writer ); 
			writer.Write( (int) 0 ); 
		} 

		public override void Deserialize(GenericReader reader) 
		{ 
			base.Deserialize( reader ); 
			int version = reader.ReadInt(); 
		} 
	} 
} 
