using System;
using Server.Misc;

namespace Server.Items
{
	public class HelmOfTheBard : LeatherCap 
	{ 
		private SkillMod m_SkillMod0; 
		private SkillMod m_SkillMod1; 
		private SkillMod m_SkillMod2; 
		private SkillMod m_SkillMod3; 
		private StatMod m_StatMod0; 

		public override int BasePhysicalResistance{ get{ return 10; } }
		public override int BaseFireResistance{ get{ return 12; } }
		public override int BaseColdResistance{ get{ return 12; } }
		public override int BasePoisonResistance{ get{ return 8; } }
		public override int BaseEnergyResistance{ get{ return 8; } }

		public override int InitMinHits{ get{ return 255; } }
		public override int InitMaxHits{ get{ return 255; } }

		public override int AosStrReq{ get{ return 0; } }

		public override int OldStrReq{ get{ return 0; } }
		public override int OldDexBonus{ get{ return 0; } }

		[Constructable] 
		public HelmOfTheBard()
		{ 
			Name = "Helm Of The Bard"; 
			Hue = 1160; 
			DefineMods();
			Attributes.NightSight = 1;

			Attributes.SpellDamage = 4;
			Attributes.CastRecovery = 1;
			Attributes.CastSpeed = 1;
			Attributes.LowerManaCost = 3;
			Attributes.RegenHits = 1;
			Attributes.Luck = 40;
			ArmorAttributes.SelfRepair = 5;
		} 

		private void DefineMods()
		{
			m_SkillMod0 = new DefaultSkillMod( SkillName.Discordance, true, 2 ); 
			m_SkillMod1 = new DefaultSkillMod( SkillName.Provocation, true, 2 ); 
			m_SkillMod2 = new DefaultSkillMod( SkillName.Peacemaking, true, 2 ); 
			m_SkillMod3 = new DefaultSkillMod( SkillName.Musicianship, true, 2 ); 
			m_StatMod0 = new StatMod( StatType.Dex, "BardHelm", 2, TimeSpan.Zero ); 
		}

		private void SetMods( Mobile wearer )
		{			
			wearer.AddSkillMod( m_SkillMod0 ); 
			wearer.AddSkillMod( m_SkillMod1 ); 
			wearer.AddSkillMod( m_SkillMod2 ); 
			wearer.AddSkillMod( m_SkillMod3 ); 
			wearer.AddStatMod( m_StatMod0 ); 
		}

		public override bool OnEquip( Mobile from ) 
		{ 
			SetMods( from );
			return true;  
		} 

		public override void OnRemoved( object parent ) 
		{ 
			if ( parent is Mobile ) 
			{ 
				Mobile m = (Mobile)parent;
				m.RemoveStatMod( "BardHelm" ); 

				if ( m.Hits > m.HitsMax )
					m.Hits = m.HitsMax; 

				if ( m_SkillMod0 != null ) 
					m_SkillMod0.Remove(); 

				if ( m_SkillMod1 != null ) 
					m_SkillMod1.Remove(); 

				if ( m_SkillMod2 != null ) 
					m_SkillMod2.Remove(); 

				if ( m_SkillMod3 != null ) 
					m_SkillMod3.Remove(); 
			} 
		} 

		public override void OnSingleClick( Mobile from ) 
		{ 
			this.LabelTo( from, Name ); 
		} 

		public HelmOfTheBard( Serial serial ) : base( serial ) 
		{ 
			DefineMods();
			
			if ( Parent != null && this.Parent is Mobile ) 
				SetMods( (Mobile)Parent );
		} 

		public override void Serialize( GenericWriter writer ) 
		{ 
			base.Serialize( writer ); 
			writer.Write( (int) 0 ); 
		} 

		public override void Deserialize(GenericReader reader) 
		{ 
			base.Deserialize( reader ); 
			int version = reader.ReadInt(); 
		} 
	} 
} 
