using System;
using Server.Misc;

namespace Server.Items
{

	public class LegsOfTheWarrior : RingmailLegs 
	{ 
		private SkillMod m_SkillMod0; 
		private SkillMod m_SkillMod1; 
		private SkillMod m_SkillMod2; 
		private SkillMod m_SkillMod3; 
		private StatMod m_StatMod0; 
		
		public override ArmorMeditationAllowance DefMedAllowance{ get{ return ArmorMeditationAllowance.All; } }

		public override int BasePhysicalResistance{ get{ return 12; } }
		public override int BaseFireResistance{ get{ return 10; } }
		public override int BaseColdResistance{ get{ return 10; } }
		public override int BasePoisonResistance{ get{ return 10; } }
		public override int BaseEnergyResistance{ get{ return 8; } }

		public override int InitMinHits{ get{ return 255; } }
		public override int InitMaxHits{ get{ return 255; } }

		public override int AosStrReq{ get{ return 0; } }

		public override int OldStrReq{ get{ return 0; } }
		public override int OldDexBonus{ get{ return 0; } }

		[Constructable] 
		public LegsOfTheWarrior()
		{ 
			Name = "Legs Of The Warrior"; 
			Hue = 0x485; 
			DefineMods();

			Attributes.RegenStam = 2;
			Attributes.DefendChance = 3;
			Attributes.AttackChance = 5;
			Attributes.WeaponDamage = 8;
			Attributes.WeaponSpeed = 8;
			Attributes.ReflectPhysical = 4;
			ArmorAttributes.SelfRepair = 5;
		} 

		private void DefineMods()
		{
			m_SkillMod0 = new DefaultSkillMod( SkillName.Tactics, true, 2 ); 
			m_SkillMod1 = new DefaultSkillMod( SkillName.Healing, true, 2 ); 
			m_SkillMod2 = new DefaultSkillMod( SkillName.Anatomy, true, 2 ); 
			m_SkillMod3 = new DefaultSkillMod( SkillName.Focus, true, 2 ); 
			m_StatMod0 = new StatMod( StatType.Str, "WarLegs", 2, TimeSpan.Zero ); 
		}

		private void SetMods( Mobile wearer )
		{			
			wearer.AddSkillMod( m_SkillMod0 ); 
			wearer.AddSkillMod( m_SkillMod1 ); 
			wearer.AddSkillMod( m_SkillMod2 ); 
			wearer.AddSkillMod( m_SkillMod3 ); 
			wearer.AddStatMod( m_StatMod0 ); 
		}

		public override bool OnEquip( Mobile from ) 
		{ 
			SetMods( from );
			return true;  
		} 

		public override void OnRemoved( object parent ) 
		{ 
			if ( parent is Mobile ) 
			{ 
				Mobile m = (Mobile)parent;
				m.RemoveStatMod( "WarLegs" ); 

				if ( m.Hits > m.HitsMax )
					m.Hits = m.HitsMax; 

				if ( m_SkillMod0 != null ) 
					m_SkillMod0.Remove(); 

				if ( m_SkillMod1 != null ) 
					m_SkillMod1.Remove(); 

				if ( m_SkillMod2 != null ) 
					m_SkillMod2.Remove(); 

				if ( m_SkillMod3 != null ) 
					m_SkillMod3.Remove(); 
			} 
		} 

		public override void OnSingleClick( Mobile from ) 
		{ 
			this.LabelTo( from, Name ); 
		} 

		public LegsOfTheWarrior( Serial serial ) : base( serial ) 
		{ 
			DefineMods();
			
			if ( Parent != null && this.Parent is Mobile ) 
				SetMods( (Mobile)Parent );
		} 

		public override void Serialize( GenericWriter writer ) 
		{ 
			base.Serialize( writer ); 
			writer.Write( (int) 0 ); 
		} 

		public override void Deserialize(GenericReader reader) 
		{ 
			base.Deserialize( reader ); 
			int version = reader.ReadInt(); 
		} 
	} 
} 
