/*
    _________________________________
 -=(_)_______________________________)=-
   /   .   .   . ____  . ___      _/
  /~ /    /   / /     / /   )2007 /
 (~ (____(___/ (____ / /___/     (
  \ ----------------------------- \
   \    lucidnagual@charter.net    \
    \_     ===================      \
     \   -Owner of "The Conjuring"-  \
      \_     ===================     ~\
       )      Lucid's Mega Pack        )
      /~    The Mother Load v1.1     _/
    _/_______________________________/
 -=(_)_______________________________)=-

 */
using System;
using System.IO;
using System.Collections;
using Server.Items;
using Server.Misc;
using Server.Gumps;
using Server.ContextMenus;
using Server.Mobiles;
using Server.Accounting;
using Server.ACC.CM;
using Server.Enums;
using Server.LucidNagual;
using Server.Messages;


namespace Server.Items
{
	public class RavensRaceStone : Item
	{
		[Constructable]
		public RavensRaceStone() : base( 0xED4 )
		{
			Movable = false;
			Hue = 1170;
			Name = "Ravens Race Stone";
		}
		
		public override void OnDoubleClick( Mobile from )
		{
			if ( from.Mounted )
			{
				from.SendLocalizedMessage( 1042146 ); // You can not use this while mounted.
			}
			else
			{
				if ( DataCenter.EnableRaceSystem )
					from.SendGump( new RavensRaceGump(from, RavensRacePage.Info ) );
				else
					from.SendMessage( "Sorry the race system is disabled." );
			}
		}
		
		public RavensRaceStone( Serial serial ) : base( serial )
		{
		}
		
		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 ); // version
		}
		
		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
	
	public enum RavensRacePage
	{
		Info,
		Drow,
		Angel,
		Elf,
		Vampire,
		Dwarf,
		Orc,
		Human,
	}
	
	public class RavensRaceGump : Gump
	{
		private Mobile m_From;
		private RavensRacePage m_Page;
		
		private const int Blanco = 0xFFFFFF;
		private const int Azul = 0x8080FF;
		
		public void AddPageButton( int x, int y, int buttonID, string text, RavensRacePage page, params RavensRacePage[] subpage )
		{
			bool seleccionado = ( m_Page == page );
			
			for ( int i = 0; !seleccionado && i < subpage.Length; ++i )
				seleccionado = ( m_Page == subpage[i] );
			
			AddButton( x, y - 1, seleccionado ? 4006 : 4005, 4007, buttonID, GumpButtonType.Reply, 0 );
			AddHtml( x + 35, y, 200, 20, Color( text, seleccionado ? Azul : Blanco ), false, false );
		}
		
		public void AddButtonLabeled( int x, int y, int buttonID, string text )
		{
			AddButton( x, y - 1, 4005, 4007, buttonID, GumpButtonType.Reply, 0 );
			AddHtml( x + 35, y, 240, 20, Color( text, Blanco ), false, false );
		}
		
		public int GetButtonID( int type, int index )
		{
			return 1 + (index * 15) + type;
		}
		
		public string Color( string text, int color )
		{
			return String.Format( "<BASEFONT COLOR=#{0:X6}>{1}</BASEFONT>", color, text );
		}
		
		public RavensRaceGump ( Mobile from, RavensRacePage page) : base ( 40, 45 )
		{
			from.CloseGump( typeof( RavensRaceGump ) );
			
			m_From = from;
			m_Page = page;
			Closable = false;
			Dragable = true;
			from.Frozen = false;
			
			AddPage( 0 );
			AddBackground( 0, 0, 531, 360, 5054 );
			AddAlphaRegion( 10, 10, 511, 340 );
			
			//Barra Escoge tu raza
			AddImageTiled( 11, 37, 149, 19, 0x52 );
			AddImageTiled( 11, 37, 148, 18, 0xBBC );
			AddLabel( 35, 37, 0, "Choose Your Race" ); // choose your race
			
			//Barra menu de seleccion
			AddImageTiled( 11, 11, 509, 21, 0x52 );
			AddImageTiled( 11, 11, 507, 19, 0xBBC );
			AddLabel( 165, 11, 0, "Race Selection Menu" ); // Race selection menu
			
			//Lineas divisorias
			AddImageTiled( 10, 32, 511, 5, 5058 );
			AddImageTiled( 10, 287, 511, 5, 5058 );
			AddImageTiled( 160, 37, 5, 250, 5058 );
			
			AddPageButton( 40, 75, GetButtonID( 0, 0 ), "Initial Page", RavensRacePage.Info ); //Initial Page
			AddPageButton( 40, 100, GetButtonID( 0, 1 ), "Drow", RavensRacePage.Drow); // DarkElf
			AddPageButton( 40, 125, GetButtonID( 0, 2 ), "Angel", RavensRacePage.Angel); // HalfElf
			AddPageButton( 40, 150, GetButtonID( 0, 3 ), "Elf", RavensRacePage.Elf); // Elf
			AddPageButton( 40, 175, GetButtonID( 0, 4 ), "Vampire", RavensRacePage.Vampire); // Vampire
			AddPageButton( 40, 200, GetButtonID( 0, 5 ), "Dwarf", RavensRacePage.Dwarf); // Dragon
			AddPageButton( 40, 225, GetButtonID( 0, 6 ), "Orc", RavensRacePage.Orc); // Druid
			AddPageButton( 40, 250, GetButtonID( 0, 7 ), "Human", RavensRacePage.Human); // Normal
			AddButton( 490, 11, 0xFB1, 0xFB3, 7, GumpButtonType.Reply, 0 ); //dupres code for close
			AddLabel( 450, 11, 0x34, "Close" ); //dupres code
			
			
			switch ( page )
			{
				case RavensRacePage.Info:
					{
						AddHtml( 172, 40, 337, 245, String.Format("Welcome to the Ravens Race Stone. Please select your race from the menu provided."), true, true); break;
						
					}
					
				case RavensRacePage.Drow:
					{
						AddHtml( 172, 40, 337, 245, String.Format("<center>* Drow *</center>\nThe DRows are a physically beautiful race, lithe and graceful, slightly shorter than humans, and light, by human standards, for their weight. But it is a cold beauty, for the dark elves are a cruel and ruthless race, their dark skin, white hair, and finely chiselled features making them very distinctive indeed. Dark elves are wise and intelligent, They make for great treasure hunters or thieves.\n"), true, true ); // DarkElf Description
						AddPageButton( 50, 317, GetButtonID( 1, 0 ), "Back To Main Menu", RavensRacePage.Info); // Back to Main Menu
						AddButtonLabeled( 300, 317, GetButtonID( 1, 1 ), "I Want To Be Drow" ); // I want to be a DarkElf
						break;
					}
					
				case RavensRacePage.Angel:
					{
						AddHtml( 172, 40, 337, 245, String.Format("<center>* Angel *</center>\nAngels are a calm loving race. they have only one natural enemy. Orcs... Angels have been known to be massvely good or massvely bad... Angels were created in a holy gods image. They were to remain that way until Mondain was released...\n"), true, true ); // HalfElf Description
						AddPageButton( 50, 317, GetButtonID( 1, 0 ), "Back To The Main Menu", RavensRacePage.Info); // Back to Main Menu
						AddButtonLabeled( 300, 317, GetButtonID( 1, 2 ), "I want to be a Angel!" ); // I want to be a Angel
						break;
					}
					
				case RavensRacePage.Elf:
					{
						AddHtml( 172, 40, 337, 245, String.Format("<center>* Elf *</center>\nThe Elves are one of the oldest races in the lands. Tall and willowy, possessed of great physical beauty and remarkable intelligence, elves tend to be blonde haired and blue eyes. There are exceptions to this rule, but the bearing and innate nobility of a elf is unmistakeable. There sense of Magic makes them great mages.\n "), true, true ); // Elf Description
						AddPageButton( 50, 317, GetButtonID( 1, 0 ), "Back ToMain Menu", RavensRacePage.Info); // Back to Main Menu
						AddButtonLabeled( 300, 317, GetButtonID( 1, 3 ), "I Want To Be A Elf" ); // I want to be a Elf
						break;
					}
					
				case RavensRacePage.Vampire:
					{
						AddHtml( 172, 40, 337, 245, String.Format("<center>* Vampire *</center>\nThe vampire are very light skinned race, They are a race which survive by sucking blood out of they're prey.\n "), true, true ); // Vampire Description
						AddPageButton( 50, 317, GetButtonID( 1, 0 ), "Back To The Main Menu", RavensRacePage.Info); // Back to Main Menu
						AddButtonLabeled( 300, 317, GetButtonID( 1, 4 ), "I Want To Be A Vampire" ); // I want to be a Vampire
						break;
					}
					
				case RavensRacePage.Dwarf:
					{
						AddHtml( 172, 40, 337, 245, String.Format("<center>* Dwarf *</center>\nThe Dwarfs are Fickle and enigmatic creatures, Dwarfs are Large, Short and Strong seeming. Despite their changeable and complex natures, these magical crafters usually seem to have benign motivations and desires. Their magical nature makes many of them talented crafters, while their sense of honor leads a few of them to battle.\n"), true, true ); // Dragon Description
						AddPageButton( 50, 317, GetButtonID( 1, 0 ), "Back To The Main Menu", RavensRacePage.Info); // Back to Main Menu
						AddButtonLabeled( 300, 317, GetButtonID( 1, 5 ), "I Want To Be A Dwarf" ); // I want to be a HalfDwarf
						break;
					}
					
				case RavensRacePage.Orc:
					{
						AddHtml( 172, 40, 337, 245, String.Format("<center>* Orc *</center>\nOrcs Are Brutes Of Brittania Orcs Are Natural Armorers and Warriors... 1 vs 1 combat with an Orc means certain death. Armor made by orcs is evil looking and jagged... orcs are know to have spikes in their armor so that they can tackle and enemy and kill them without working as had to swing a war hammer.\n"), true, true ); // Druid Description
						AddPageButton( 50, 317, GetButtonID( 1, 0 ), "Back To The Main Menu", RavensRacePage.Info); // Back to Main Menu
						AddButtonLabeled( 300, 317, GetButtonID( 1, 6 ), "I Want To Be A Orc" ); // I want to be a Orc
						break;
					}
					
				case RavensRacePage.Human:
					{
						AddHtml( 172, 40, 337, 245, String.Format("<center>* Human *</center>\nThis Selection Will Turn You Back To Normal. Without The Title And Hue Change.\n"), true, true ); // Druid Description
						AddPageButton( 50, 317, GetButtonID( 1, 0 ), "Back To The Main Menu", RavensRacePage.Info); // Back to Main Menu
						AddButtonLabeled( 300, 317, GetButtonID( 1, 7 ), "I Want To Be A Human" ); // I want to be a Human
						break;
					}
			}
			
		}
		
		public override void OnResponse( Server.Network.NetState sender, RelayInfo info )
		{
			int val = info.ButtonID - 1;
			
			if ( val < 0 )
				return;
			
			Mobile from = m_From;
			from.Frozen = false;
			
			PlayerMobile pm = from as PlayerMobile;
			PlayerModule module = pm.PlayerModule;
			
			int type = val % 15;
			int index = val / 15;

			switch ( type )
			{
				case 0:
					{
						RavensRacePage page;
						
						switch ( index )
						{
								case 0: page = RavensRacePage.Info; break;
								case 1: page = RavensRacePage.Drow; break;
								case 2: page = RavensRacePage.Angel; break;
								case 3: page = RavensRacePage.Elf; break;
								case 4: page = RavensRacePage.Vampire; break;
								case 5: page = RavensRacePage.Dwarf; break;
								case 6: page = RavensRacePage.Orc; break;
								case 7: page = RavensRacePage.Human; break;
								default: return;
						}
						
						from.SendGump( new RavensRaceGump( from, page) );
						break;
					}
					
				case 1:
					{
						switch ( index )
						{
							case 0:
								{
									from.SendGump( new RavensRaceGump( from, RavensRacePage.Info) );
									break;
								}
								
							case 1:
								{
									m_From.SendMessage( "you are now a Drow!" );
									m_From.SendMessage( MsgCenter.Msg[9].Text );
									m_From.Hue = 1109;
									m_From.Title = " the Drow";
									module.PlayerRaces = PlayerRaces.Drow;
									break;
								}
								
							case 2:
								{
									m_From.SendMessage( "you are now a Angel!" );
									m_From.SendMessage( MsgCenter.Msg[9].Text );
									m_From.Hue = 1153;
									m_From.Title = " the Angel";
									module.PlayerRaces = PlayerRaces.Angel;
									break;
								}
								
							case 3:
								{
									m_From.SendMessage( "you are now a Elf" );
									m_From.SendMessage( MsgCenter.Msg[9].Text );
									m_From.Hue = 600;
									m_From.Title = " the Elf";
									module.PlayerRaces = PlayerRaces.Elf;
									break;
								}
								
							case 4:
								{
									m_From.SendMessage( "you are now a Vampire!" );
									m_From.SendMessage( MsgCenter.Msg[9].Text );
									m_From.Hue = 33918;
									m_From.Title = " the Vampire";
									module.PlayerRaces = PlayerRaces.Vampire;
									break;
								}
								
							case 5:
								{
									m_From.SendMessage( "you are now a Dwarf!" );
									m_From.SendMessage( MsgCenter.Msg[9].Text );
									m_From.Hue = 1149;
									m_From.Title = " the Dwarf";
									module.PlayerRaces = PlayerRaces.Dwarf;
									break;
								}
								
							case 6:
								{
									m_From.SendMessage( "you are now a Orc!" );
									m_From.SendMessage( MsgCenter.Msg[9].Text );
									m_From.Hue = 1155;
									m_From.Title = " the Orc";
									module.PlayerRaces = PlayerRaces.Orc;
									
									m_From.HairItemID = 0/*0x203D*/; // Pony Tail
									m_From.HairHue = 0/*0x22*/;
									
									m_From.FacialHairItemID = 0/*0x203D*/; // Pony Tail
									m_From.FacialHairHue = 0/*0x22*/;
									
									//if ( from.HairInfo != null )
									//	from.Hair.Delete();
									//if ( from.Beard != null )
									//	from.Beard.Delete();
									//Add morphing ball.
									from.EquipItem( new OrcFace() );
									break;
								}
								
							case 7:
								{
									m_From.SendMessage( "you are now a Human!" );
									m_From.SendMessage( MsgCenter.Msg[9].Text );
									m_From.Hue = 33800;
									m_From.Title = "";
									module.PlayerRaces = PlayerRaces.Human;
									if ( from.Female && from.Alive )
									{
										from.BodyValue = 401;
										from.SendGump( new HairRestylingGump( from ) );
									}
									else
									{
										from.BodyValue = 400;
										from.SendGump( new BeardRestylingGump( from ) );
										from.SendGump( new HairRestylingGump( from ) );
									}
									break;
								}
						}
						break;
					}
			}
			
		}
	}
}

