using System;
using Server.Targeting;
using Server.Network;


namespace Server.Spells.Second
{
	public class AgilitySpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
		                                                "Agility", "Ex Uus",
		                                                SpellCircle.Second,
		                                                212,
		                                                9061,
		                                                Reagent.Bloodmoss,
		                                                Reagent.MandrakeRoot
		                                               );
		
		public AgilitySpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}
		
		public override void OnCast()
		{
			if ( Server.Spells.AllSpells.CheckRestriction( Caster, 2 ) == false )
				return;
			
			Caster.Target = new InternalTarget( this );
		}
		
		public void Target( Mobile m )
		{
			if ( !Caster.CanSee( m ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckBSequence( m ) )
			{
				SpellHelper.Turn( Caster, m );
				
				SpellHelper.AddStatBonus( Caster, m, StatType.Dex );
				
				m.FixedParticles( 0x375A, 10, 15, 5010, EffectLayer.Waist );
				m.PlaySound( 0x28E );
			}
			
			FinishSequence();
		}
		
		private class InternalTarget : Target
		{
			private AgilitySpell m_Owner;
			
			public InternalTarget( AgilitySpell owner ) : base( 12, false, TargetFlags.Beneficial )
			{
				m_Owner = owner;
			}
			
			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is Mobile )
				{
					m_Owner.Target( (Mobile)o );
				}
			}
			
			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}
